
from logging import Logger
from logging import getLogger


from click import secho

from buildlackey.Environment import Environment

PYPI_PUSH: str = 'twine upload  dist/*'


class ProductionPush(Environment):
    def __init__(self):
        super().__init__()
        self.logger: Logger = getLogger(__name__)

    def execute(self):

        secho(f'{PYPI_PUSH}')
        status = self._runCommand(PYPI_PUSH)
        secho(f'{status=}')




from logging import Logger
from logging import getLogger

from pathlib import Path

from click import ClickException
from click import secho

from buildlackey.Environment import Environment

# noinspection SpellCheckingInspection
BUILD_WHEEL:   str = 'python -m build --sdist --wheel'


class Package(Environment):
    def __init__(self, inputFile: str):

        super().__init__()
        self.logger: Logger = getLogger(__name__)

        self._inputFile: str = inputFile

    def execute(self):

        self._changeToProjectRoot()

        if self._inputFile is None:
            secho(f'{BUILD_WHEEL}')
            status: int = self._runCommand(BUILD_WHEEL)
            secho(f'{status=}')

            CHECK_PACKAGE: str = 'twine check dist/*'
            secho(f'{CHECK_PACKAGE}')
            status = self._runCommand(CHECK_PACKAGE)
            secho(f'{status=}')
        else:
            path: Path = Path(self._inputFile)
            if path.exists() is True:
                with path.open(mode='r') as fd:
                    cmd: str = fd.readline()
                    while cmd != '':
                        secho(f'{cmd}')
                        status = self._runCommand(f'{cmd}')
                        if status != 0:
                            exit(status)
                        cmd = fd.readline()
            else:
                raise ClickException(f'No such file: {self._inputFile}')




from logging import Logger
from logging import getLogger

from click import secho

from buildlackey.Environment import Environment


class RunMypy(Environment):
    
    def __init__(self, packageName: str):
        super().__init__()
        self.logger:       Logger = getLogger(__name__)
        self._packageName: str    = packageName

    def execute(self):
        self._changeToProjectRoot()

        if self._packageName is None:
            packageName: str = self._projectDirectory
        else:
            packageName = self._packageName

        # noinspection SpellCheckingInspection
        cmd: str = f'mypy --config-file .mypi.ini --pretty --no-color-output --show-error-codes --check-untyped-defs  {packageName} tests'
        secho(f'{cmd}')

        status: int = self._runCommand(command=cmd)
        secho(f'{status=}')




from logging import Logger
from logging import getLogger

from click import secho

from buildlackey.Environment import Environment

DELETE_DIST_BUILD:       str = 'rm -rfv dist build'
DELETE_GENERAL_EGG_INFO: str = "find . -type d -name '*'.egg-info -delete"
DELETE_LOG_FILES:        str = 'find . -type f -name "*.log"      -delete'
DELETE_EGGS:             str = 'rm -rfv .eggs'


class Cleanup(Environment):
    
    def __init__(self, packageName: str):
        super().__init__()
        self.logger:       Logger = getLogger(__name__)
        self._packageName: str    = packageName

    def execute(self):

        self._changeToProjectRoot()

        secho(f'{DELETE_DIST_BUILD}')
        status: int = self._runCommand(DELETE_DIST_BUILD)
        secho(f'{status=}')

        secho(f'{DELETE_GENERAL_EGG_INFO}')
        status = self._runCommand(DELETE_GENERAL_EGG_INFO)
        secho(f'{status=}')

        secho(f'{DELETE_LOG_FILES}')
        status = self._runCommand(DELETE_LOG_FILES)
        secho(f'{status=}')

        secho(f'{DELETE_EGGS}')
        status = self._runCommand(DELETE_EGGS)
        secho(f'{status=}')

        if self._packageName is None:
            packageName: str = self._projectDirectory
        else:
            packageName = self._packageName

        PROJECT_EGG_INFO: str = f'rm -rfv {packageName}.egg-info'
        secho(f'{PROJECT_EGG_INFO}')
        status = self._runCommand(PROJECT_EGG_INFO)
        secho(f'{status=}')




from logging import Logger
from logging import getLogger

from os import linesep as osLineSep

from pathlib import Path

from click import ClickException
from click import secho

from buildlackey.Environment import Environment
from buildlackey.PythonWarnings import PythonWarnings

STATUS_NO_SUCH_PATH: int = 23

DEFAULT_MODULE_NAME: str = 'tests.TestAll'
PYTHON_CLI:          str = 'python3'


class RunTests(Environment):
    
    def __init__(self, inputFile: str, warning: str):
        super().__init__()

        self.logger: Logger = getLogger(__name__)

        self._inputFile: str = inputFile
        if warning is None:
            self._warning: PythonWarnings = PythonWarnings.IGNORE
        else:
            try:
                self._warning = PythonWarnings(warning)
            except ValueError:
                raise ClickException(f'Invalid warning type: {warning}')

    def execute(self):

        if self.validProjectsBase is True and self.validProjectDirectory() is True:
            self._changeToProjectRoot()

        if self._inputFile is None:
            defaultCmd: str = f'{PYTHON_CLI} -W{self._warning.value} -m tests.TestAll'
            secho(f'{defaultCmd}')
            status: int = self._runCommand(command=f'python3 -W{self._warning.value} -m {DEFAULT_MODULE_NAME}')
            secho(f'{status=}')
        else:
            path: Path = Path(self._inputFile)
            if path.exists() is True:
                with path.open(mode='r') as fd:
                    moduleName: str = fd.readline()

                    while moduleName != '':
                        if moduleName != osLineSep and not moduleName.startswith('#'):
                            # noinspection SpellCheckingInspection
                            cmd: str = f'{PYTHON_CLI} -W{self._warning.value} -m {moduleName}'
                            secho(f'{cmd}')
                            status = self._runCommand(command=cmd)
                            if status != 0:
                                exit(status)
                        moduleName = fd.readline()
            else:
                secho(f'No such file: {self._inputFile}')
                exit(STATUS_NO_SUCH_PATH)




import logging
import logging.config

from importlib.resources import files
from importlib.abc import Traversable

from json import load as jsonLoad

from click import command
from click import option
from click import version_option

from buildlackey import __version__ as version

from buildlackey.commands.Cleanup import Cleanup
from buildlackey.commands.Package import Package
from buildlackey.commands.ProductionPush import ProductionPush
from buildlackey.commands.RunMypy import RunMypy
from buildlackey.commands.RunTests import RunTests


RESOURCES_PACKAGE_NAME:       str = 'buildlackey.resources'
JSON_LOGGING_CONFIG_FILENAME: str = "loggingConfiguration.json"

"""
Put in type ignore because of strange error on that appeared on 8.1.4

buildlackey/Commands.py:80: error: Argument 1 has incompatible type "Callable[[], Any]"; expected <nothing>  [arg-type]
    @command
"""


def setUpLogging():
    """
    """
    traversable: Traversable = files(RESOURCES_PACKAGE_NAME) / JSON_LOGGING_CONFIG_FILENAME

    loggingConfigFilename: str = str(traversable)

    with open(loggingConfigFilename, 'r') as loggingConfigurationFile:
        configurationDictionary = jsonLoad(loggingConfigurationFile)

    logging.config.dictConfig(configurationDictionary)
    logging.logProcesses = False
    logging.logThreads = False


@command
@version_option(version=f'{version}', message='%(prog)s version %(version)s')
@option('--input-file', '-i', required=False,   help='Use input file to list the unit tests to execute')
@option('--warning',    '-w', required=False,   help='Use this option to control Python warnings')
def runtests(input_file: str, warning: str):
    """
    \b
    Runs the unit tests for the project specified by the environment variables listed below;
    \b
    Use the -i/--input-file option to list a set of module names to execute as your
    unit tests

    Legal values for -w/--warning are:

    \b
        default
        error
        always
        module
        once
        ignore
    \b
    Environment Variables

        PROJECTS_BASE -  The local directory where the python projects are based
        PROJECT       -  The name of the project;  It should be a directory name

    \b
    \b
    However, if one or the other is not defined the command assumes it is executing in a CI
    environment and thus the current working directory is the project base directory.

    By default, buildlackey runs the module named tests.TestAll
    """
    setUpLogging()
    runTests: RunTests = RunTests(inputFile=input_file, warning=warning)

    runTests.execute()


@command
@version_option(version=f'{version}', message='%(prog)s version %(version)s')
@option('--package-name', '-p', required=False, help='Use this option when the package name does not match the project name')
def cleanup(package_name: str):
    """
    \b
    Clean the build artifacts for the project specified by the following environment variables
    \b
        PROJECTS_BASE -  The local directory where the python projects are based
        PROJECT       -  The name of the project;  It should be a directory name

    PROJECT is overridden if the developer specifies a package name
    """
    setUpLogging()
    clean: Cleanup = Cleanup(packageName=package_name)

    clean.execute()


@command
@version_option(version=f'{version}', message='%(prog)s version %(version)s')
@option('--package-name', '-p', required=False, help='Use this option when the package name does not match the project name')
def runmypy(package_name: str):
    """
    \b
    Runs the mypy checks for the project specified by the following environment variables
    \b
        PROJECTS_BASE -  The local directory where the python projects are based
        PROJECT       -  The name of the project;  It should be a directory name

    PROJECT is overridden if the developer specifies a package name
    """
    runMyPy: RunMypy = RunMypy(packageName=package_name)
    runMyPy.execute()


@command
@version_option(version=f'{version}', message='%(prog)s version %(version)s')
@option('--input-file', '-i', required=False,   help='Use input file to specify a set of commands to execute')
def package(input_file: str):
    """
    \b
    Creates the deployable for the project specified by the environment variables listed below
    \b
    Use the -i/--input-file option to specify a set of custom commands to execute to build
    your deployable

    Environment Variables
    \b
        PROJECTS_BASE -  The local directory where the python projects are based
        PROJECT       -  The name of the project;  It should be a directory name

    """
    setUpLogging()
    pkg: Package = Package(inputFile=input_file)

    pkg.execute()


@command
@version_option(version=f'{version}', message='%(prog)s version %(version)s')
def prodpush():
    """
    \b
    Pushes the deployable to pypi.  The project is specified by the following environment variables
    \b
        PROJECTS_BASE -  The local directory where the python projects are based
        PROJECT       -  The name of the project;  It should be a directory name
    """
    productionPush: ProductionPush = ProductionPush()
    productionPush.execute()


if __name__ == "__main__":
    runtests([])
    # noinspection SpellCheckingInspection
    # runmypy(['-p', 'codeallyadvanced'])
    # runtests(['-i', 'tests/unittest.txt'])
    # cleanup(['--help'])
    # deploy(['--help'])




import pathlib
from setuptools import setup

from buildlackey import __version__ as version
# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README  = (HERE / "README.md").read_text()
LICENSE = (HERE / 'LICENSE').read_text()

setup(
    name="buildlackey",
    version=version,
    author='Humberto A. Sanchez II',
    author_email='humberto.a.sanchez.ii@gmail.com',
    maintainer='Humberto A. Sanchez II',
    maintainer_email='humberto.a.sanchez.ii@gmail.com',
    description='Project Maintenance Scripts',
    long_description=README,
    long_description_content_type="text/markdown",
    license=LICENSE,
    url="https://github.com/buildlackey",
    packages=[
        'buildlackey',
        'buildlackey.commands',
        'buildlackey.exceptions',
        'buildlackey.resources',
    ],
    package_data={
        'buildlackey.resources': ['loggingConfiguration.json', 'version.txt'],
    },

    install_requires=[
        'codeallybasic==0.5.2',
        'click~=8.1.7',
    ],
    entry_points={
        "console_scripts": [
            "runtests=buildlackey.Commands:runtests",
            "cleanup=buildlackey.Commands:cleanup",
            "runmypy=buildlackey.Commands:runmypy",
            "package=buildlackey.Commands:package",
            "prodpush=buildlackey.Commands:prodpush",
        ]
    },
)




from logging import Logger
from logging import getLogger
from subprocess import CompletedProcess

from os import chdir

from os import environ as osEnvironment
from os import sep as osSep

from subprocess import run as subProcessRun

from click import clear
from click import secho


class Environment:
    """

    """
    ENV_PROJECTS_BASE: str = 'PROJECTS_BASE'
    ENV_PROJECT:       str = 'PROJECT'

    def __init__(self):

        self.ebLogger: Logger = getLogger(__name__)

        self._projectsBase:     str = ''
        self._projectDirectory: str = ''

        try:
            self._projectsBase = osEnvironment[Environment.ENV_PROJECTS_BASE]
        except KeyError:
            self.ebLogger.info(f'Project Base not set')
        try:
            self._projectDirectory = osEnvironment[Environment.ENV_PROJECT]
        except KeyError:
            self.ebLogger.info(f'Project Directory not set')

        clear()
        secho(f'projects_base={self._projectsBase}', color=True, reverse=True)
        secho(f'project={self._projectDirectory}', color=True, reverse=True)
        secho('')

    @property
    def projectsBase(self) -> str:
        return self._projectsBase

    @property
    def projectDirectory(self) -> str:
        return self._projectDirectory

    @property
    def validProjectsBase(self) -> bool:
        if self._projectsBase == '':
            return False
        else:
            return True

    def validProjectDirectory(self) -> bool:
        if self._projectDirectory == '':
            return False
        else:
            return True

    def _runCommand(self,  command: str) -> int:

        cp: CompletedProcess = subProcessRun([command], shell=True, capture_output=True, text=True, check=False)

        secho(f'{cp.stdout}')
        if cp.returncode != 0:
            secho(f'{cp.stderr}', bold=True)
        return cp.returncode

    def _changeToProjectRoot(self):

        fullPath: str = f'{self._projectsBase}{osSep}{self._projectDirectory}'
        chdir(fullPath)




from enum import Enum


class PythonWarnings(Enum):
    DEFAULT: str = 'default'    # Warn once per call location
    ERROR:   str = 'error'      # Convert to exceptions
    ALWAYS:  str = 'always'     # Warn every time
    MODULE:  str = 'module'     # Warn once per calling module
    ONCE:    str = 'once'       # Warn once per Python process
    IGNORE:  str = 'ignore'     # Never warn
